"""
Support for displaying the minimal and the maximal value.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/sensor.min_max/
"""
import asyncio
import logging

import voluptuous as vol

import homeassistant.helpers.config_validation as cv
from homeassistant.components.sensor import PLATFORM_SCHEMA
from homeassistant.const import (
    CONF_NAME, STATE_UNKNOWN, CONF_TYPE, ATTR_UNIT_OF_MEASUREMENT)
from homeassistant.core import callback
from homeassistant.helpers.entity import Entity
from homeassistant.helpers.event import async_track_state_change

_LOGGER = logging.getLogger(__name__)

ATTR_MIN_VALUE = 'min_value'
ATTR_MAX_VALUE = 'max_value'
ATTR_COUNT_SENSORS = 'count_sensors'
ATTR_MEAN = 'mean'

ATTR_TO_PROPERTY = [
    ATTR_COUNT_SENSORS,
    ATTR_MAX_VALUE,
    ATTR_MEAN,
    ATTR_MIN_VALUE,
]

CONF_ENTITY_IDS = 'entity_ids'

DEFAULT_NAME = 'Min/Max/Avg Sensor'

ICON = 'mdi:calculator'

SENSOR_TYPES = {
    ATTR_MIN_VALUE: 'min',
    ATTR_MAX_VALUE: 'max',
    ATTR_MEAN: 'mean',
}

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Optional(CONF_TYPE, default=SENSOR_TYPES[ATTR_MAX_VALUE]):
        vol.All(cv.string, vol.In(SENSOR_TYPES.values())),
    vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
    vol.Required(CONF_ENTITY_IDS): cv.entity_ids,
})


@asyncio.coroutine
def async_setup_platform(hass, config, async_add_devices, discovery_info=None):
    """Set up the min/max/mean sensor."""
    entity_ids = config.get(CONF_ENTITY_IDS)
    name = config.get(CONF_NAME)
    sensor_type = config.get(CONF_TYPE)

    hass.loop.create_task(async_add_devices(
        [MinMaxSensor(hass, entity_ids, name, sensor_type)], True))
    return True


class MinMaxSensor(Entity):
    """Representation of a min/max sensor."""

    def __init__(self, hass, entity_ids, name, sensor_type):
        """Initialize the min/max sensor."""
        self._hass = hass
        self._entity_ids = entity_ids
        self._sensor_type = sensor_type
        self._name = '{} {}'.format(
            name, next(v for k, v in SENSOR_TYPES.items()
                       if self._sensor_type == v))
        self._unit_of_measurement = None
        self.min_value = self.max_value = self.mean = STATE_UNKNOWN
        self.count_sensors = len(self._entity_ids)
        self.states = {}

        @callback
        # pylint: disable=invalid-name
        def async_min_max_sensor_state_listener(entity, old_state, new_state):
            """Called when the sensor changes state."""
            if new_state.state is None or new_state.state in STATE_UNKNOWN:
                return

            if self._unit_of_measurement is None:
                self._unit_of_measurement = new_state.attributes.get(
                    ATTR_UNIT_OF_MEASUREMENT)

            if self._unit_of_measurement != new_state.attributes.get(
                    ATTR_UNIT_OF_MEASUREMENT):
                _LOGGER.warning("Units of measurement do not match")
                return
            try:
                self.states[entity] = float(new_state.state)
            except ValueError:
                _LOGGER.warning("Unable to store state. "
                                "Only numerical states are supported")

            hass.loop.create_task(self.async_update_ha_state(True))

        async_track_state_change(
            hass, entity_ids, async_min_max_sensor_state_listener)

    @property
    def name(self):
        """Return the name of the sensor."""
        return self._name

    @property
    def state(self):
        """Return the state of the sensor."""
        return getattr(self, next(
            k for k, v in SENSOR_TYPES.items() if self._sensor_type == v))

    @property
    def unit_of_measurement(self):
        """Return the unit the value is expressed in."""
        return self._unit_of_measurement

    @property
    def should_poll(self):
        """No polling needed."""
        return False

    @property
    def device_state_attributes(self):
        """Return the state attributes of the sensor."""
        state_attr = {
            attr: getattr(self, attr) for attr
            in ATTR_TO_PROPERTY if getattr(self, attr) is not None
        }
        return state_attr

    @property
    def icon(self):
        """Return the icon to use in the frontend, if any."""
        return ICON

    @asyncio.coroutine
    def async_update(self):
        """Get the latest data and updates the states."""
        sensor_values = [self.states[k] for k in self._entity_ids
                         if k in self.states]
        if len(sensor_values) == self.count_sensors:
            self.min_value = min(sensor_values)
            self.max_value = max(sensor_values)
            self.mean = round(sum(sensor_values) / self.count_sensors, 2)
        else:
            self.min_value = self.max_value = self.mean = STATE_UNKNOWN



"""
Support for Envisalink-based alarm control panels (Honeywell/DSC).

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/alarm_control_panel.envisalink/
"""
import logging

import homeassistant.components.alarm_control_panel as alarm
from homeassistant.components.envisalink import (
    EVL_CONTROLLER, EnvisalinkDevice, PARTITION_SCHEMA, CONF_CODE, CONF_PANIC,
    CONF_PARTITIONNAME, SIGNAL_PARTITION_UPDATE, SIGNAL_KEYPAD_UPDATE)
from homeassistant.const import (
    STATE_ALARM_ARMED_AWAY, STATE_ALARM_ARMED_HOME, STATE_ALARM_DISARMED,
    STATE_UNKNOWN, STATE_ALARM_TRIGGERED)

_LOGGER = logging.getLogger(__name__)

DEPENDENCIES = ['envisalink']


# pylint: disable=unused-argument
def setup_platform(hass, config, add_devices, discovery_info=None):
    """Perform the setup for Envisalink alarm panels."""
    _configured_partitions = discovery_info['partitions']
    _code = discovery_info[CONF_CODE]
    _panic_type = discovery_info[CONF_PANIC]
    for part_num in _configured_partitions:
        _device_config_data = PARTITION_SCHEMA(
            _configured_partitions[part_num])
        _device = EnvisalinkAlarm(
            part_num,
            _device_config_data[CONF_PARTITIONNAME],
            _code,
            _panic_type,
            EVL_CONTROLLER.alarm_state['partition'][part_num],
            EVL_CONTROLLER)
        add_devices([_device])

    return True


class EnvisalinkAlarm(EnvisalinkDevice, alarm.AlarmControlPanel):
    """Representation of an Envisalink-based alarm panel."""

    def __init__(self, partition_number, alarm_name, code, panic_type, info,
                 controller):
        """Initialize the alarm panel."""
        from pydispatch import dispatcher
        self._partition_number = partition_number
        self._code = code
        self._panic_type = panic_type
        _LOGGER.debug("Setting up alarm: %s", alarm_name)
        EnvisalinkDevice.__init__(self, alarm_name, info, controller)
        dispatcher.connect(
            self._update_callback, signal=SIGNAL_PARTITION_UPDATE,
            sender=dispatcher.Any)
        dispatcher.connect(
            self._update_callback, signal=SIGNAL_KEYPAD_UPDATE,
            sender=dispatcher.Any)

    def _update_callback(self, partition):
        """Update HA state, if needed."""
        if partition is None or int(partition) == self._partition_number:
            self.hass.async_add_job(self.update_ha_state)

    @property
    def code_format(self):
        """The characters if code is defined."""
        return self._code

    @property
    def state(self):
        """Return the state of the device."""
        if self._info['status']['alarm']:
            return STATE_ALARM_TRIGGERED
        elif self._info['status']['armed_away']:
            return STATE_ALARM_ARMED_AWAY
        elif self._info['status']['armed_stay']:
            return STATE_ALARM_ARMED_HOME
        elif self._info['status']['alpha']:
            return STATE_ALARM_DISARMED
        else:
            return STATE_UNKNOWN

    def alarm_disarm(self, code=None):
        """Send disarm command."""
        if self._code:
            EVL_CONTROLLER.disarm_partition(
                str(code), self._partition_number)

    def alarm_arm_home(self, code=None):
        """Send arm home command."""
        if self._code:
            EVL_CONTROLLER.arm_stay_partition(
                str(code), self._partition_number)

    def alarm_arm_away(self, code=None):
        """Send arm away command."""
        if self._code:
            EVL_CONTROLLER.arm_away_partition(
                str(code), self._partition_number)

    def alarm_trigger(self, code=None):
        """Alarm trigger command. Will be used to trigger a panic alarm."""
        if self._code:
            EVL_CONTROLLER.panic_alarm(self._panic_type)



"""
Support for ZoneMinder Sensors.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/sensor.zoneminder/
"""
import logging

from homeassistant.const import STATE_UNKNOWN
from homeassistant.helpers.entity import Entity
import homeassistant.components.zoneminder as zoneminder

_LOGGER = logging.getLogger(__name__)

DEPENDENCIES = ['zoneminder']


def setup_platform(hass, config, add_devices, discovery_info=None):
    """Set up the ZoneMinder sensor platform."""
    sensors = []

    monitors = zoneminder.get_state('api/monitors.json')
    for i in monitors['monitors']:
        sensors.append(
            ZMSensorMonitors(int(i['Monitor']['Id']), i['Monitor']['Name'])
        )
        sensors.append(
            ZMSensorEvents(int(i['Monitor']['Id']), i['Monitor']['Name'])
        )

    add_devices(sensors)


class ZMSensorMonitors(Entity):
    """Get the status of each ZoneMinder monitor."""

    def __init__(self, monitor_id, monitor_name):
        """Initiate monitor sensor."""
        self._monitor_id = monitor_id
        self._monitor_name = monitor_name
        self._state = None

    @property
    def name(self):
        """Return the name of the sensor."""
        return '{} Status'.format(self._monitor_name)

    @property
    def state(self):
        """Return the state of the sensor."""
        return self._state

    def update(self):
        """Update the sensor."""
        monitor = zoneminder.get_state(
            'api/monitors/%i.json' % self._monitor_id
        )
        if monitor['monitor']['Monitor']['Function'] is None:
            self._state = STATE_UNKNOWN
        else:
            self._state = monitor['monitor']['Monitor']['Function']


class ZMSensorEvents(Entity):
    """Get the number of events for each monitor."""

    def __init__(self, monitor_id, monitor_name):
        """Initiate event sensor."""
        self._monitor_id = monitor_id
        self._monitor_name = monitor_name
        self._state = None

    @property
    def name(self):
        """Return the name of the sensor."""
        return '{} Events'.format(self._monitor_name)

    @property
    def unit_of_measurement(self):
        """Return the unit of measurement of this entity, if any."""
        return 'Events'

    @property
    def state(self):
        """Return the state of the sensor."""
        return self._state

    def update(self):
        """Update the sensor."""
        event = zoneminder.get_state(
            'api/events/index/MonitorId:%i.json' % self._monitor_id
        )

        self._state = event['pagination']['count']



"""
Support for ZWave climate devices.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/climate.zwave/
"""
# Because we do not compile openzwave on CI
# pylint: disable=import-error
import logging
from homeassistant.components.climate import DOMAIN
from homeassistant.components.climate import (
    ClimateDevice, ATTR_OPERATION_MODE)
from homeassistant.components.zwave import ZWaveDeviceEntity
from homeassistant.components import zwave
from homeassistant.const import (
    TEMP_CELSIUS, TEMP_FAHRENHEIT, ATTR_TEMPERATURE)

_LOGGER = logging.getLogger(__name__)

CONF_NAME = 'name'
DEFAULT_NAME = 'ZWave Climate'

REMOTEC = 0x5254
REMOTEC_ZXT_120 = 0x8377
REMOTEC_ZXT_120_THERMOSTAT = (REMOTEC, REMOTEC_ZXT_120)

HORSTMANN = 0x0059
HORSTMANN_HRT4_ZW = 0x3
HORSTMANN_HRT4_ZW_THERMOSTAT = (HORSTMANN, HORSTMANN_HRT4_ZW)

WORKAROUND_ZXT_120 = 'zxt_120'
WORKAROUND_HRT4_ZW = 'hrt4_zw'

DEVICE_MAPPINGS = {
    REMOTEC_ZXT_120_THERMOSTAT: WORKAROUND_ZXT_120,
    HORSTMANN_HRT4_ZW_THERMOSTAT: WORKAROUND_HRT4_ZW
}

SET_TEMP_TO_INDEX = {
    'Heat': 1,
    'Cool': 2,
    'Auto': 3,
    'Aux Heat': 4,
    'Resume': 5,
    'Fan Only': 6,
    'Furnace': 7,
    'Dry Air': 8,
    'Moist Air': 9,
    'Auto Changeover': 10,
    'Heat Econ': 11,
    'Cool Econ': 12,
    'Away': 13,
    'Unknown': 14
}


def setup_platform(hass, config, add_devices, discovery_info=None):
    """Setup the ZWave Climate devices."""
    if discovery_info is None or zwave.NETWORK is None:
        _LOGGER.debug("No discovery_info=%s or no NETWORK=%s",
                      discovery_info, zwave.NETWORK)
        return
    temp_unit = hass.config.units.temperature_unit
    node = zwave.NETWORK.nodes[discovery_info[zwave.const.ATTR_NODE_ID]]
    value = node.values[discovery_info[zwave.const.ATTR_VALUE_ID]]
    value.set_change_verified(False)
    add_devices([ZWaveClimate(value, temp_unit)])
    _LOGGER.debug("discovery_info=%s and zwave.NETWORK=%s",
                  discovery_info, zwave.NETWORK)


class ZWaveClimate(ZWaveDeviceEntity, ClimateDevice):
    """Represents a ZWave Climate device."""

    def __init__(self, value, temp_unit):
        """Initialize the zwave climate device."""
        from openzwave.network import ZWaveNetwork
        from pydispatch import dispatcher
        ZWaveDeviceEntity.__init__(self, value, DOMAIN)
        self._node = value.node
        self._target_temperature = None
        self._current_temperature = None
        self._current_operation = None
        self._operation_list = None
        self._operating_state = None
        self._current_fan_mode = None
        self._fan_list = None
        self._current_swing_mode = None
        self._swing_list = None
        self._unit = temp_unit
        self._index_operation = None
        _LOGGER.debug("temp_unit is %s", self._unit)
        self._zxt_120 = None
        self._hrt4_zw = None
        self.update_properties()
        # register listener
        dispatcher.connect(
            self.value_changed, ZWaveNetwork.SIGNAL_VALUE_CHANGED)
        # Make sure that we have values for the key before converting to int
        if (value.node.manufacturer_id.strip() and
                value.node.product_id.strip()):
            specific_sensor_key = (int(value.node.manufacturer_id, 16),
                                   int(value.node.product_id, 16))
            if specific_sensor_key in DEVICE_MAPPINGS:
                if DEVICE_MAPPINGS[specific_sensor_key] == WORKAROUND_ZXT_120:
                    _LOGGER.debug("Remotec ZXT-120 Zwave Thermostat"
                                  " workaround")
                    self._zxt_120 = 1
                if DEVICE_MAPPINGS[specific_sensor_key] == WORKAROUND_HRT4_ZW:
                    _LOGGER.debug("Horstmann HRT4-ZW Zwave Thermostat"
                                  " workaround")
                    self._hrt4_zw = 1

    def value_changed(self, value):
        """Called when a value has changed on the network."""
        if self._value.value_id == value.value_id or \
           self._value.node == value.node:
            self.update_properties()
            self.update_ha_state()
            _LOGGER.debug("Value changed on network %s", value)

    def update_properties(self):
        """Callback on data change for the registered node/value pair."""
        # Operation Mode
        for value in self._node.get_values(
                class_id=zwave.const.COMMAND_CLASS_THERMOSTAT_MODE).values():
            self._current_operation = value.data
            self._index_operation = SET_TEMP_TO_INDEX.get(
                self._current_operation)
            self._operation_list = list(value.data_items)
            _LOGGER.debug("self._operation_list=%s", self._operation_list)
            _LOGGER.debug("self._current_operation=%s",
                          self._current_operation)
        # Current Temp
        for value in (self._node.get_values(
                class_id=zwave.const.COMMAND_CLASS_SENSOR_MULTILEVEL)
                      .values()):
            if value.label == 'Temperature':
                self._current_temperature = int(value.data)
                self._unit = value.units
        # Fan Mode
        for value in (self._node.get_values(
                class_id=zwave.const.COMMAND_CLASS_THERMOSTAT_FAN_MODE)
                      .values()):
            self._current_fan_mode = value.data
            self._fan_list = list(value.data_items)
            _LOGGER.debug("self._fan_list=%s", self._fan_list)
            _LOGGER.debug("self._current_fan_mode=%s",
                          self._current_fan_mode)
        # Swing mode
        if self._zxt_120 == 1:
            for value in (self._node.get_values(
                    class_id=zwave.const.COMMAND_CLASS_CONFIGURATION)
                          .values()):
                if value.command_class == \
                   zwave.const.COMMAND_CLASS_CONFIGURATION and \
                   value.index == 33:
                    self._current_swing_mode = value.data
                    self._swing_list = list(value.data_items)
                    _LOGGER.debug("self._swing_list=%s", self._swing_list)
                    _LOGGER.debug("self._current_swing_mode=%s",
                                  self._current_swing_mode)
        # Set point
        for value in (self._node.get_values(
                class_id=zwave.const.COMMAND_CLASS_THERMOSTAT_SETPOINT)
                      .values()):
            if value.data == 0:
                _LOGGER.debug("Setpoint is 0, setting default to "
                              "current_temperature=%s",
                              self._current_temperature)
                self._target_temperature = int(self._current_temperature)
                break
            if self.current_operation is not None and \
               self.current_operation != 'Off':
                if self._index_operation != value.index:
                    continue
                if self._zxt_120:
                    break
                self._target_temperature = int(value.data)
                break
            _LOGGER.debug("Device can't set setpoint based on operation mode."
                          " Defaulting to index=1")
            self._target_temperature = int(value.data)
        # Operating state
        for value in (self._node.get_values(
                class_id=zwave.const.COMMAND_CLASS_THERMOSTAT_OPERATING_STATE)
                      .values()):
            self._operating_state = value.data

    @property
    def should_poll(self):
        """No polling on ZWave."""
        return False

    @property
    def current_fan_mode(self):
        """Return the fan speed set."""
        return self._current_fan_mode

    @property
    def fan_list(self):
        """List of available fan modes."""
        return self._fan_list

    @property
    def current_swing_mode(self):
        """Return the swing mode set."""
        return self._current_swing_mode

    @property
    def swing_list(self):
        """List of available swing modes."""
        return self._swing_list

    @property
    def temperature_unit(self):
        """Return the unit of measurement."""
        if self._unit == 'C':
            return TEMP_CELSIUS
        elif self._unit == 'F':
            return TEMP_FAHRENHEIT
        else:
            return self._unit

    @property
    def current_temperature(self):
        """Return the current temperature."""
        return self._current_temperature

    @property
    def current_operation(self):
        """Return the current operation mode."""
        return self._current_operation

    @property
    def operation_list(self):
        """List of available operation modes."""
        return self._operation_list

    @property
    def target_temperature(self):
        """Return the temperature we try to reach."""
        return self._target_temperature

    def set_temperature(self, **kwargs):
        """Set new target temperature."""
        if kwargs.get(ATTR_TEMPERATURE) is not None:
            temperature = kwargs.get(ATTR_TEMPERATURE)
        else:
            return
        operation_mode = kwargs.get(ATTR_OPERATION_MODE)
        _LOGGER.debug("set_temperature operation_mode=%s", operation_mode)

        for value in (self._node.get_values(
                class_id=zwave.const.COMMAND_CLASS_THERMOSTAT_SETPOINT)
                      .values()):
            if operation_mode is not None:
                setpoint_mode = SET_TEMP_TO_INDEX.get(operation_mode)
                if value.index != setpoint_mode:
                    continue
                _LOGGER.debug("setpoint_mode=%s", setpoint_mode)
                value.data = temperature
                break

            if self.current_operation is not None:
                if self._hrt4_zw and self.current_operation == 'Off':
                    # HRT4-ZW can change setpoint when off.
                    value.data = int(temperature)
                if self._index_operation != value.index:
                    continue
                _LOGGER.debug("self._index_operation=%s and"
                              " self._current_operation=%s",
                              self._index_operation,
                              self._current_operation)
                if self._zxt_120:
                    _LOGGER.debug("zxt_120: Setting new setpoint for %s, "
                                  " operation=%s, temp=%s",
                                  self._index_operation,
                                  self._current_operation, temperature)
                    # ZXT-120 does not support get setpoint
                    self._target_temperature = temperature
                    # ZXT-120 responds only to whole int
                    value.data = round(temperature, 0)
                    self.update_ha_state()
                    break
                else:
                    _LOGGER.debug("Setting new setpoint for %s, "
                                  "operation=%s, temp=%s",
                                  self._index_operation,
                                  self._current_operation, temperature)
                    value.data = temperature
                    break
            else:
                _LOGGER.debug("Setting new setpoint for no known "
                              "operation mode. Index=1 and "
                              "temperature=%s", temperature)
                value.data = temperature
                break

    def set_fan_mode(self, fan):
        """Set new target fan mode."""
        for value in (self._node.get_values(
                class_id=zwave.const.COMMAND_CLASS_THERMOSTAT_FAN_MODE).
                      values()):
            if value.command_class == \
               zwave.const.COMMAND_CLASS_THERMOSTAT_FAN_MODE and \
               value.index == 0:
                value.data = bytes(fan, 'utf-8')
                break

    def set_operation_mode(self, operation_mode):
        """Set new target operation mode."""
        for value in self._node.get_values(
                class_id=zwave.const.COMMAND_CLASS_THERMOSTAT_MODE).values():
            if value.command_class == \
               zwave.const.COMMAND_CLASS_THERMOSTAT_MODE and value.index == 0:
                value.data = bytes(operation_mode, 'utf-8')
                break

    def set_swing_mode(self, swing_mode):
        """Set new target swing mode."""
        if self._zxt_120 == 1:
            for value in self._node.get_values(
                    class_id=zwave.const.COMMAND_CLASS_CONFIGURATION).values():
                if value.command_class == \
                   zwave.const.COMMAND_CLASS_CONFIGURATION and \
                   value.index == 33:
                    value.data = bytes(swing_mode, 'utf-8')
                    break

    @property
    def device_state_attributes(self):
        """Return the device specific state attributes."""
        if self._operating_state:
            return {
                "operating_state": self._operating_state,
            }
        else:
            return {}



"""
Support for Neato Connected Vaccums.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/switch.neato/
"""
import time
import logging
from datetime import timedelta
from urllib.error import HTTPError
from requests.exceptions import HTTPError as req_HTTPError

import voluptuous as vol

from homeassistant.const import (CONF_PASSWORD, CONF_USERNAME, STATE_OFF,
                                 STATE_ON, STATE_UNAVAILABLE)
from homeassistant.helpers.entity import ToggleEntity
import homeassistant.helpers.config_validation as cv

_LOGGER = logging.getLogger(__name__)

REQUIREMENTS = ['https://github.com/jabesq/pybotvac/archive/v0.0.1.zip'
                '#pybotvac==0.0.1']

MIN_TIME_BETWEEN_SCANS = timedelta(seconds=10)
MIN_TIME_BETWEEN_FORCED_SCANS = timedelta(milliseconds=100)

MIN_TIME_TO_WAIT = timedelta(seconds=10)
MIN_TIME_TO_LOCK_UPDATE = 10

SWITCH_TYPES = {
    'clean': ['Clean']
}

DOMAIN = 'neato'

CONFIG_SCHEMA = vol.Schema({
    DOMAIN: vol.Schema({
        vol.Required(CONF_USERNAME): cv.string,
        vol.Required(CONF_PASSWORD): cv.string,
    })
}, extra=vol.ALLOW_EXTRA)


def setup_platform(hass, config, add_devices, discovery_info=None):
    """Setup the Neato platform."""
    from pybotvac import Account

    try:
        auth = Account(config[CONF_USERNAME], config[CONF_PASSWORD])
    except HTTPError:
        _LOGGER.error("Unable to connect to Neato API")
        return False

    dev = []
    for robot in auth.robots:
        for type_name in SWITCH_TYPES:
            dev.append(NeatoConnectedSwitch(robot, type_name))
    add_devices(dev)


class NeatoConnectedSwitch(ToggleEntity):
    """Neato Connected Switch (clean)."""

    def __init__(self, robot, switch_type):
        """Initialize the Neato Connected switch."""
        self.type = switch_type
        self.robot = robot
        self.lock = False
        self.last_lock_time = None
        self.graceful_state = False
        self._state = None

    def lock_update(self):
        """Lock the update since Neato clean takes some time to start."""
        if self.is_update_locked():
            return
        self.lock = True
        self.last_lock_time = time.time()

    def reset_update_lock(self):
        """Reset the update lock."""
        self.lock = False
        self.last_lock_time = None

    def set_graceful_lock(self, state):
        """Set the graceful state."""
        self.graceful_state = state
        self.reset_update_lock()
        self.lock_update()

    def is_update_locked(self):
        """Check if the update method is locked."""
        if self.last_lock_time is None:
            return False

        if time.time() - self.last_lock_time >= MIN_TIME_TO_LOCK_UPDATE:
            self.last_lock_time = None
            return False

        return True

    @property
    def state(self):
        """Return the state."""
        if not self._state:
            return STATE_UNAVAILABLE
        if not self._state['availableCommands']['start'] and \
           not self._state['availableCommands']['stop'] and \
           not self._state['availableCommands']['pause'] and \
           not self._state['availableCommands']['resume'] and \
           not self._state['availableCommands']['goToBase']:
            return STATE_UNAVAILABLE
        return STATE_ON if self.is_on else STATE_OFF

    @property
    def name(self):
        """Return the name of the sensor."""
        return self.robot.name + ' ' + SWITCH_TYPES[self.type][0]

    @property
    def is_on(self):
        """Return true if device is on."""
        if self.is_update_locked():
            return self.graceful_state
        if self._state['action'] == 1 and self._state['state'] == 2:
            return True
        return False

    def turn_on(self, **kwargs):
        """Turn the device on."""
        self.set_graceful_lock(True)
        self.robot.start_cleaning()

    def turn_off(self, **kwargs):
        """Turn the device off (Return Robot to base)."""
        self.robot.pause_cleaning()
        time.sleep(1)
        self.robot.send_to_base()

    def update(self):
        """Refresh Robot state from Neato API."""
        try:
            self._state = self.robot.state
        except req_HTTPError:
            _LOGGER.error("Unable to retrieve to Robot State.")
            self._state = None
            return False
